@@ -5,7 +5,7 @@
 #  open source, and has the attribution requirements (GPL Section 7) at
 #  https://statnet.org/attribution .
 #
-#  Copyright 2007-2022 Statnet Commons
+#  Copyright 2007-2024 Statnet Commons
 ################################################################################
 .autodetect_dep_warn <- local({
   warned <- c()
@@ -156,7 +156,7 @@ handle.controls <- function(myname, ...){
 
   if(exists("match.arg.pars", parent.frame()))
     for(arg in get("match.arg.pars", parent.frame()))
-      control[arg] <- list(match.arg(control[[arg]][1], eval(formal.args[[arg]])))
+      control[arg] <- list(match.arg(control[[arg]][1], eval(formal.args[[arg]], parent.frame())))
 
   control
 }
@@ -361,7 +361,8 @@ as.control.list.list <- function(x, FUN=NULL, unflat=TRUE, ...){
           l[aname] <- list(x[[aname]])
           xnames_unused <<- setdiff(xnames_unused, aname)
         }else if(is.call(aval <- args[[aname]]) && startsWith(as.character(aval[[1]]), "control.")){ # A control list not supplied directly: process recursively.
-          l[aname] <- list(unflat(get(as.character(aval[[1]]), pos=environment(f), mode="function")))
+          subargs <- unflat(get(as.character(aval[[1]]), pos=environment(f), mode="function"))
+          if(length(subargs)) l[aname] <- list(subargs)
         }
       # Otherwise, leave blank.
 
